import static org.junit.Assert.*;

import org.junit.Test;

import tokensAndTrees.Dot;
import tokensAndTrees.Id;
import tokensAndTrees.Left;
import tokensAndTrees.Right;
import tokensAndTrees.Token;
import tokensAndTrees.Tree;


public class TestParser {

	private Tree run(Token[] in) throws ParseException {
		TokenSource t = new ArrayTokenSource(in) ;
		AbstractParser p = new Parser() ;
		return p.startPrime(t) ;
	}

	@Test public void test_null() {
		Token[] in = new Token[]{ Left.object, Right.object } ;
		try {
			Tree tree = run(in);
			assertEquals("Null", tree.toString());
		} catch (ParseException e) {
			fail() ;
		}
	}

	@Test public void test_id() {
		Token[] in = new Token[]{ new Id("fred") } ;
		try {
			Tree tree = run(in);
			assertEquals("Id(fred)", tree.toString());
		} catch (ParseException e) {
			fail() ;
		}
	}

	@Test public void test_list_1() {
		Token[] in = new Token[]{ Left.object, new Id("fred"), Right.object } ;
		try {
			Tree tree = run(in);
			assertEquals("Cons(Id(fred), Null)", tree.toString());
		} catch (ParseException e) {
			fail() ;
		}
	}

	@Test public void test_list_n() {
		Token[] in = new Token[]{ Left.object, new Id("bill"), new Id("percy"), new Id("fred"),
				new Id("george"), new Id("ron"), new Id("ginny"), Right.object } ;
		try {
			Tree tree = run(in);
			assertEquals("Cons(Id(bill), Cons(Id(percy), Cons(Id(fred), Cons(Id(george), Cons(Id(ron), Cons(Id(ginny), Null))))))",
					tree.toString());
		} catch (ParseException e) {
			fail() ;
		}
	}

	@Test public void test_dot_1() {
		Token[] in = new Token[]{ Left.object, new Id("harry"), Dot.object, new Id("hermione"), Right.object } ;
		try {
			Tree tree = run(in);
			assertEquals("Cons(Id(harry), Id(hermione))", tree.toString());
		} catch (ParseException e) {
			fail() ;
		}
	}

	@Test public void test_dot_many() {
		Token[] in = new Token[]{ Left.object, new Id("I"),  new Id("am"), new Id("lord"), Dot.object, new Id("Voldemort"), Right.object } ;
		try {
			Tree tree = run(in);
			assertEquals("Cons(Id(I), Cons(Id(am), Cons(Id(lord), Id(Voldemort))))", tree.toString()) ;
		} catch (ParseException e) {
			fail() ;
		}
	}

	@Test public void test_nested() {
		Token[] in = new Token[]{ Left.object, Left.object, Right.object,
											   Left.object, new Id("dumbledore"), Right.object,
				                               Left.object, new Id("macgonagle"), Dot.object, new Id("snape"), Right.object,
				                  Right.object } ;
		try {
			Tree tree = run(in);
			assertEquals("Cons(Null, Cons(Cons(Id(dumbledore), Null), Cons(Cons(Id(macgonagle), Id(snape)), Null)))", tree.toString()) ;
		} catch (ParseException e) {
			fail() ;
		}
	}

	@Test public void test_missing_close() {
		Token[] in = new Token[]{ Left.object, } ;
		try {
			Tree tree = run(in);
			fail() ;
		} catch (ParseException e) {
			System.out.println(e.getMessage()) ;
		}
	}

	@Test public void test_missing_open() {
		Token[] in = new Token[]{ Right.object } ;
		try {
			Tree tree = run(in);
			fail() ;
		} catch (ParseException e) {
			System.out.println(e.getMessage()) ;
		}
	}

	@Test public void test_extra_close() {
		Token[] in = new Token[]{ Left.object, Right.object, Right.object } ;
		try {
			Tree tree = run(in);
			fail() ;
		} catch (ParseException e) {
			System.out.println(e.getMessage()) ;
		}
	}

	@Test public void test_extra_after_dot() {
		Token[] in = new Token[]{ Left.object, new Id("a"), Dot.object, new Id("b"), new Id("c"), Right.object } ;
		try {
			Tree tree = run(in);
			fail() ;
		} catch (ParseException e) {
			System.out.println(e.getMessage()) ;
		}
	}

	@Test public void test_extra_id() {
		Token[] in = new Token[]{ new Id("a"), new Id("b") } ;
		try {
			Tree tree = run(in);
			fail() ;
		} catch (ParseException e) {
			System.out.println(e.getMessage()) ;
		}
	}

	@Test public void test_nothing_before_dot() {
		Token[] in = new Token[]{ Left.object, Dot.object, new Id("a"), Right.object } ;
		try {
			Tree tree = run(in);
			fail() ;
		} catch (ParseException e) {
			System.out.println(e.getMessage()) ;
		}
	}

}
